# -*- coding: utf-8 -*-
import os
import tkinter
from tkinter.filedialog import Open, Directory, SaveAs

# Find the cureent working directory.
here = os.getcwd()


def conditional_kwargs(**nkwargs):
    """Returns function with conditionally supplied kwargs.

    This decorator is intended for use with higher level functions that
    act more like wrappers for other functions or classes.
    """
    def decorator(some_function):
        def wrapper(nkwargs=nkwargs, *args, **kwargs):
            for k, v in nkwargs.items():
                if k not in kwargs:
                    kwargs[k] = v
                else:
                    pass
            return some_function(*args, **kwargs)
        return wrapper
    return decorator


def root_topmost():
    """Return root as a withdrawn topmost window.
    """
    root = tkinter.Tk()
    # Hide the main window.
    root.withdraw()
    root.call('wm', 'attributes', ".", '-topmost', True)
    return root


class filedialog(object):
    """Jupyter Notebook freindly tkinter filedialogs."""
    # The following functions have been modified from the following;

    # https://github.com/python/cpython/blob/3.6/Lib/tkinter/filedialog.py

    @staticmethod
    @conditional_kwargs(parent=root_topmost(), initialdir=here)
    def askopenfilename(**options):
        "Ask for a filename to open."
        return Open(**options).show()

    @staticmethod
    @conditional_kwargs(parent=root_topmost(), initialdir=here)
    def asksaveasfilename(**options):
        "Ask for a filename to save as."
        return SaveAs(**options).show()

    @staticmethod
    @conditional_kwargs(parent=root_topmost(), initialdir=here)
    def askopenfilenames(**options):
        """Ask for multiple filenames to open.
        Returns a list of filenames or empty list if
        cancel button selected
        """
        options["multiple"] = 1
        return Open(**options).show()

    @staticmethod
    @conditional_kwargs(parent=root_topmost(), initialdir=here)
    def askopenfile(mode="r", **options):
        "Ask for a filename to open, and returned the opened file"

        filename = Open(**options).show()
        if filename:
            return open(filename, mode)
        return None

    @staticmethod
    @conditional_kwargs(parent=root_topmost(), initialdir=here)
    def askopenfiles(mode="r", **options):
        """Ask for multiple filenames and return the open file
        objects
        returns a list of open file objects or an empty list if
        cancel selected
        """

        files = askopenfilenames(**options)
        if files:
            ofiles = []
            for filename in files:
                ofiles.append(open(filename, mode))
            files = ofiles
        return files

    @staticmethod
    @conditional_kwargs(parent=root_topmost(), initialdir=here)
    def asksaveasfile(mode="w", **options):
        "Ask for a filename to save as, and returned the opened file"

        filename = SaveAs(**options).show()
        if filename:
            return open(filename, mode)
        return None

    @staticmethod
    @conditional_kwargs(parent=root_topmost(), initialdir=here)
    def askdirectory(**options):
        "Ask for a directory, and return the file name"
        return Directory(**options).show()



# -*- coding: utf-8 -*-
import xlrd
import pandas as pd

class CLITools(object):
    """Command Line Interface Tools.
    """
    @staticmethod
    def list_choice_cli(inp, message="Choose from the following:\n"):
        """Return the users selection from a list.

        Parameters
        ----------
        inp : list

        message : string

        Returns
        -------
        result : string
        """
        show = ["{}) {}\n".format(n, i) for n,i in enumerate(inp)]
        show = ''.join(show)
        show = message+show
        print(show)
        selection = int(input())
        result = inp[selection]
        return result

    @classmethod
    def read_excel_cli(cls, file_name, **kwargs):
        """Return a DataFrame from the selected sheet of an Excel file.

        Parameters
        ----------
        file_name : string

        Returns
        -------
        result : DataFrame

        """
        book = xlrd.open_workbook(file_name)
        if book.nsheets > 1:
            selected_sheet = cls.list_choice_cli(book.sheet_names())
            result = pd.read_excel(file_name,
                                   sheet_name=selected_sheet,
                                   **kwargs)
        else:
            result = pd.read_excel(file_name, **kwargs)
        return result



# -*- coding: utf-8 -*-
import os
import xlrd
import pandas as pd
from .cli_tools import CLITools
from ..jtkinter import filedialog

class DataLoader(object):
    """
    """

    def __init__(self, **kwargs):
        self.source_file_name = None
        self.raw = None
        self.buffer = filedialog.askopenfile()

        try:
            self.source_file_name = self.buffer.name

        except AttributeError as e:
            print(e.args[0])

        self.source_file_ext = os.path.splitext(self.source_file_name)[-1]

        if self.source_file_ext == ".xls" or self.source_file_ext == ".xlsx":

            try:
                # Excel files are so special that need to be read by a different function.
                # NOTE: Only the first sheet in the file will be read.
                # self.raw = pd.read_excel(self.buffer.name)
                self.raw = read_excel_cli(self.buffer.name, **kwargs)

            except ValueError as e:
                print(e.args[0])

        else:
            try:
                # This should work with everything else.
                self.raw = pd.read_table(self.buffer,**kwargs)

            except ValueError as e:
                print(e.args[0])
        self.buffer.close()

if __name__ == "__main__":
    dl = DataLoader()



# -*- coding: utf-8 -*-
"""
guipyter

====

Provides
--------
FIXME: ADD COPY

How to use the documentation
----------------------------
FIXME: ADD COPY

Available subpackages
---------------------

jtkinter

Utilities
---------
FIXME: ADD COPY
"""

# LICENSE
# -------
# Copyright 2018 James Draper
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files, (the software)), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions: The above copyright
# notice and this permission notice shall be included in all copies or
# substantial portions of the Software. THE SOFTWARE IS PROVIDED "AS IS",
# WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED
# TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
# NONINFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT HOLDERS BE LIABLE
# FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
# TORT OR OTHERWISE, ARISING FROM. OUT OF OR IN CONNECTION WITH THE SOFTWARE OR
# THE USE OR OTHER DEALINGS IN THE SOFTWARE.

from . import jtkinter
from .jtkinter import filedialog
from . import utils
from .utils import DataLoader
from . import notebook_tools
